//
//  DrumGeneratorViewController.m
//
//  Created by Steffan Walters on 02/01/2013.
//  Copyright (c) 2013 Steffan Walters. All rights reserved.
//

#import "DrumGeneratorViewController.h"
#import <AudioToolbox/AudioToolbox.h>
#import <AVFoundation/AVFoundation.h>
#import "drumCell.h"

//UICollectionView tutorial:http://www.adoptioncurve.net/archives/2012/09/a-simple-uicollectionview-tutorial.php

@interface DrumGeneratorViewController ()

@property float floatBeatsPerMin;    //float for the bpm (would contain 60 / beats per minute)

@property float floatDrum1Volume;   //float for drum 1 volume
@property float floatDrum2Volume;   //float for drum 2 volume

@property BOOL boolInfiniteLoop;    //bool for the playing of drums

@property (strong, nonatomic) NSString *strTableSelectedFile;     //selected file from table

@property (strong, nonatomic) AVAudioPlayer *player;    //audio players for each drum
@property (strong, nonatomic) AVAudioPlayer *player2;

@property (strong, nonatomic) NSMutableArray *marrOfFiles;  //array to store file names of saved files in

@property (strong, nonatomic) NSMutableArray *marrDrum1;    //drum arrays
@property (strong, nonatomic) NSMutableArray *marrDrum2;

@end


@implementation DrumGeneratorViewController

//see interface for brief description
@synthesize player, player2;
@synthesize marrOfFiles, marrDrum1, marrDrum2;
@synthesize tvFiles, strTableSelectedFile;
@synthesize tfTextSaveFile;
@synthesize cvDrumGrid;
@synthesize floatBeatsPerMin, tfBPM;
@synthesize floatDrum1Volume, floatDrum2Volume, sliDrum1Vol, sliDrum2Vol;
@synthesize btnLoadFile, btnDeleteFile, btnPlay, btnStop;
@synthesize boolInfiniteLoop;


#pragma mark - View lifecycle
- (void)viewDidLoad{
    
    [super viewDidLoad];
    boolInfiniteLoop = true;
    floatBeatsPerMin = 1; //initialise bpm
    
    //Initialise the drum arrays
    marrDrum1 = [[NSMutableArray alloc]init];
    marrDrum2 = [[NSMutableArray alloc]init];
    
    for(int i = 0; i<16; i++){
        [marrDrum1 addObject:@"0"];
        [marrDrum2 addObject:@"0"];
    }
    
    //UICollectionView views
    [self.cvDrumGrid registerClass:[drumCell class] forCellWithReuseIdentifier:@"Cell"];
    
    // Set up avaudioplayers for the drums
    NSString *soundFilePath = [[NSBundle mainBundle] pathForResource:@"drum1" ofType:@"wav"];
    NSURL *soundFileURL = [NSURL fileURLWithPath:soundFilePath];
    player = [[AVAudioPlayer alloc] initWithContentsOfURL:soundFileURL error:nil];
    player.numberOfLoops = 0;
    floatDrum1Volume = 0.5;
    player.volume = 0.5;
    [player prepareToPlay];
    
    NSString *soundFilePath2 = [[NSBundle mainBundle] pathForResource:@"drum2" ofType:@"wav"];
    NSURL *soundFileURL2 = [NSURL fileURLWithPath:soundFilePath2];
    player2 = [[AVAudioPlayer alloc] initWithContentsOfURL:soundFileURL2 error:nil];
    player2.numberOfLoops = 0;
    floatDrum2Volume = 0.5;
    player2.volume = 0.5;
    [player2 prepareToPlay];
    
    //Find files in the directory folder of the app and put these contents in arrayoffiles
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSFileManager *filemgr = [NSFileManager defaultManager];
    marrOfFiles = [filemgr contentsOfDirectoryAtPath:[paths objectAtIndex:0] error:nil];
    
    //Display the saved files onto the terminal
    int count = [marrOfFiles count];
    for (int i = 0; i < count; i++){
        NSLog (@"%@", [marrOfFiles objectAtIndex: i]);
    }
}

- (void)didReceiveMemoryWarning{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (IBAction)printTemo:(id)sender{
    //Print contents of the arrays on to the terminal - redundant final version
    for (int i = 0; i<16; i++) {
        NSLog(@"drum 1 = i = %i, object = %@", i, marrDrum1[i]);
        NSLog(@"drum2 = i = %i, object = %@", i, marrDrum2[i]);
    }
}




#pragma mark - Play Drums
- (IBAction)actionPlayDrums:(id)sender {
    boolInfiniteLoop = true;    //keep playing - condition for the while loop
    NSLog(@"%e", floatBeatsPerMin);
    btnPlay.alpha = 0.5;    //to ensure that you can't initiate the code below twice, disable the button
    btnPlay.enabled = false;
    
    btnStop.alpha = 1;    //enable the stop button so you can break the loop below
    btnStop.enabled = true;
    
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT,(unsigned long)NULL), ^(void) {    //do another task - allows other tasks to be performed while the code below is executed
        do {
            for (int i = 0; i < 16; i++) {   //16 beats
                
                //if the first drum is selected at the current beat, then play it
                if([[marrDrum1 objectAtIndex:i] integerValue] == 1){
                    [player play];
                }
                
                //if the second drum is selected at the current beat, then play it. etc..
                if([[marrDrum2 objectAtIndex:i] integerValue] == 1){
                    [player2 play];
                }
                
                [NSThread sleepForTimeInterval:floatBeatsPerMin];    //sleep for certain period of time to ensure beat (e.g. if beats per minutes is 60 then it would sleep for a second, 120 sleep for 0.5 etc.)
                [player stop];  //if players are still playing, then stop them (in case sample is longer than necessary)
                [player2 stop];
                
                if (!boolInfiniteLoop) {   //if booldrumloop set to false it will stop playing (stop button pressed) - break out of loop
                    break;
                }
            }
        } while (boolInfiniteLoop); //continue until user clicks the stop button
    });
}

#pragma mark - Stop Drum Playback
- (IBAction)actionStopDrums:(id)sender {  //stops the drums by setting boolInfiniteLoop to false which breaks the while loop in the actionPlayDrums method
    btnPlay.alpha = 1;    //disable the play button
    btnPlay.enabled = true;
    
    btnStop.alpha = 0.5;    //enable the stop button
    btnStop.enabled = false;

    boolInfiniteLoop = false;
}





#pragma mark - File Options
- (IBAction)actionSaveFile:(id)sender {  //Save drum beat into a file, it saves it as and array of arrays
    
    NSMutableArray *volumeArray = [[NSMutableArray alloc]init]; //allocate a space for the volume array that contains the volume of each drum and the bpm of the beat
    //create a string each to hold the bpm and volumes
    NSString *strBeatsPerMin = [[NSString alloc]initWithFormat:@"%i", tfBPM.text.integerValue];
    NSString *drum1StringVolume = [[NSString alloc]initWithFormat:@"%e",floatDrum1Volume];
    NSString *drum2StringVolume = [[NSString alloc]initWithFormat:@"%e",floatDrum2Volume];
    
    //adds the bpm and volumes to the volume array
    [volumeArray addObject:strBeatsPerMin];
    [volumeArray addObject:drum1StringVolume];
    [volumeArray addObject:drum2StringVolume];
    
    //create array for saving to the file, it contains the volume array and every drum array
    NSMutableArray *array = [[NSMutableArray alloc]init];
    
    //add the volume array and drum arrays to the array
    [array addObject:volumeArray];
    [array addObject:marrDrum1];    //array drum1 at position 0 of array array
    [array addObject:marrDrum2];
    
    //find the path of where to save the file
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    //the name of the file
    NSString *fileName = tfTextSaveFile.text;
    
    [array writeToFile:[documentsDirectory stringByAppendingPathComponent:fileName] atomically:YES]; //save the array to a file
    
    //if file not already named, then add it to the marrOfFiles and reload the table    (//if file already in the table, it will overwrite it)
    if (![marrOfFiles containsObject:tfTextSaveFile.text]) {
        [marrOfFiles addObject:tfTextSaveFile.text];
        [tvFiles reloadData];
    }
}

- (IBAction)actionLoadFile:(id)sender {
    [self actionResetGrid:sender];    //calls the actionResetGrid to reset the uicollectionview before setting it up with the contents of the loaded file
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);   //path to documents directory
    NSString *filePath = [[paths objectAtIndex:0]stringByAppendingPathComponent:strTableSelectedFile];   //path of file
    
    NSArray *fileArray = [[NSArray alloc]initWithContentsOfFile:filePath];      //the array of the drum beat
    NSMutableArray *loadedVolume = [[NSMutableArray alloc] initWithArray:fileArray[0]]; //array of the bpm and drum volumes
    NSMutableArray *loadedDrum1 = [[NSMutableArray alloc] initWithArray:fileArray[1]];  //array of drum 1
    NSMutableArray *loadedDrum2 = [[NSMutableArray alloc] initWithArray:fileArray[2]];  //etc
    
    marrDrum1 = [loadedDrum1 mutableCopy];  //mutable copy of loaded drum one, drum 1 has the same values as loadeddrum1
    marrDrum2 = [loadedDrum2 mutableCopy];  //etc
    
    //get and set the bpm and volume variables and text boxes
    NSString *bpm = [loadedVolume objectAtIndex:0];
    tfBPM.text = bpm;
    floatBeatsPerMin = 60 / [bpm floatValue];
    
    NSString *vol1 = [loadedVolume objectAtIndex:1];
    floatDrum1Volume = [vol1 floatValue];
    player.volume = floatDrum1Volume;
    sliDrum1Vol.value = (floatDrum1Volume*10);
    
    NSString *vol2 = [loadedVolume objectAtIndex:2];
    floatDrum2Volume = [vol2 floatValue];
    player2.volume = floatDrum2Volume;
    sliDrum2Vol.value = (floatDrum2Volume*10);
    
    for(int i = 0; i < 16; i++){    //16 beats
        int intDrum1IsOn = [[loadedDrum1 objectAtIndex:i] integerValue];    //returns 1 or 0 depending on value of array (1 is on, 0 is off)
        int intDrum2IsOn = [[loadedDrum2 objectAtIndex:i] integerValue];    //returns 1 or 0 depending on value of array (1 is on, 0 is off)
        
        if (intDrum1IsOn == 1) {    //if drum 1 is played at beat i
            NSIndexPath *indexPath= [NSIndexPath indexPathForRow:i inSection:0];    //get indexPath of the required view in the uicollectionview
            
            //change colour of the view at the required beat to indicate that it is on
            drumCell *cell = (drumCell *)[cvDrumGrid cellForItemAtIndexPath:indexPath];
            cell.drumView.BackgroundColor = [UIColor redColor];
        }
        
        if (intDrum2IsOn == 1) {    //if drum 2 is played at beat i
            NSIndexPath *indexPath= [NSIndexPath indexPathForRow:i inSection:1];    //get indexPath of the required view in the uicollectionview
            
            //change colour of the view at the required beat to indicate that it is on
            drumCell *cell = (drumCell *)[cvDrumGrid cellForItemAtIndexPath:indexPath];
            cell.drumView.BackgroundColor = [UIColor redColor];
        }
    }
}

- (IBAction)actionDeleteFile:(id)sender {   //create an alert asking user if they are sure they want to delete the track
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Delete Track" message:@"Are you sure that you want to delete this track?" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Delete", nil];
    
    [alert show];

}

-(void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex{
    //action called after button of a UIAction is clicked    
    if ([[alertView title] isEqual: @"Please enter a number in the range of 60 - 180"]) {   //check if it is the bpm alert box
        tfBPM.text = @"120";    //set value of bpm text box to be a valid number
        floatBeatsPerMin = 0.5;
    }else if (([[alertView title] isEqual:@"Delete Track"]) && (buttonIndex == 1)) {   //else it is the delete alert box - check if it was the delete button that was pressed
        [self delete];  //call delete method to delete fiele
    }
}

-(void) delete{
    //Get path of file to be deleted
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *filePath = [[paths objectAtIndex:0]stringByAppendingPathComponent:strTableSelectedFile];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    BOOL fileDeleted = [fileManager removeItemAtPath:filePath error:nil];   //removes file at position file path and returns yes if the file is deleted or no is the file isn't deleted
    
    [marrOfFiles removeObjectIdenticalTo:strTableSelectedFile];    //remove file from array (strTableSelectedFile is the file selected in the table)
    
    [tvFiles reloadData];  //reload table after deletion of file
    
    if (fileDeleted != YES)     //print if the file is deleted
    {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"" message:@"There was a problem - try again" delegate:self cancelButtonTitle:@"Okay" otherButtonTitles:nil, nil];
        [alert show];   //create alert and display it saying that the file was deleted
    }else{
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"" message:@"Track Deleted" delegate:self cancelButtonTitle:@"Okay" otherButtonTitles:nil, nil];
        [alert show];   //create alert and display it saying that the file was deleted
    }
    
    /*if there aren't any more files in the table, then set the load and delete button to be disabled
     so that the app doesn't try and load/delete a file that isn't there(this would caus a crash)*/
    if ([marrOfFiles count] == 0) {
        btnLoadFile.alpha = 0.5;
        btnLoadFile.enabled = false;
        btnDeleteFile.alpha = 0.5;
        btnDeleteFile.enabled = false;
    }
}


#pragma mark - reset interface
- (IBAction)actionResetGrid:(id)sender {  //reset drum arrays and the views in the uicollection view
    for (int i = 0; i < 16; i++) {      //do 16 times as there are 16 boxes representing beats
        [marrDrum1 replaceObjectAtIndex:i withObject:@"0"]; //replace 1 with a 0 in the array representing drum 1
        [self resetCollectionRow:i resetCollectionSection:0];   //call method to reset the required view in the uicollectionview
        
        [marrDrum2 replaceObjectAtIndex:i withObject:@"0"];
        [self resetCollectionRow:i resetCollectionSection:1];
    }
    
    //resets the volumes for each drum and resets the UISliders representing these volumes
    sliDrum1Vol.value = 5;
    floatDrum1Volume = 0.5;
    
    sliDrum2Vol.value = 5;
    floatDrum2Volume = 0.5;
    
    //resets the bpm value and the edit box for the bpm
    tfBPM.text = @"60";
    floatBeatsPerMin = 1;
    
    boolInfiniteLoop = false;
    
    btnPlay.alpha = 1;
    btnPlay.enabled = true;
    btnStop.alpha = 0.5;
    btnStop.enabled = false;
}





#pragma mark - Drum Sound Checks
//test the sound of the drums
- (IBAction)actionTestDrum1:(id)sender {
    [player play];
}

- (IBAction)actionTestDrum2:(id)sender {
    [player2 play];
}





- (IBAction)actionBPM:(id)sender {  //action to retrieve the value for bpm
    float textOfBox = tfBPM.text.integerValue;
    NSLog(@"%e", textOfBox);
    //if bpm is < 60 and >180 or bpm NAN then display alert asking for better bpm value
    if ((textOfBox < 60) || (textOfBox >180) || (isnan(textOfBox))) {
        NSLog(@"OH NO");
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Please enter a number in the range of 60 - 180" message:nil delegate:self cancelButtonTitle:@"Okay" otherButtonTitles:nil,nil];
        [alert show];
    }else{
        NSLog(@"%e,", textOfBox);
        floatBeatsPerMin = 60 / textOfBox;   //set sleep time for the track as 60 seconds / bpm
    }
}

- (IBAction)actionDrum1Volume:(id)sender {    //set volume for the drum player
    floatDrum1Volume = [sliDrum1Vol value];     //get value from the slider
    floatDrum1Volume = floatDrum1Volume / 10;     // /10 as it needs to be within the range 0 - 1
    player.volume = floatDrum1Volume;        //set player of drum 1 to have specific volume
}

- (IBAction)actionDrum2Volume:(id)sender {    //set volume for the drum player
    floatDrum2Volume = [sliDrum2Vol value];
    floatDrum2Volume = floatDrum2Volume / 10;
    player2.volume = floatDrum2Volume;
}





//UITableView methods
#pragma mark Table View Data Source Methods
- (NSInteger)tableView:(UITableView*)tableView numberOfRowsInSection:(NSInteger)section{
	return [marrOfFiles count]; //number of rows is equal to the number of items that have been saved (in marrOfFiles)
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView;{
	return 1;   //one section in the table
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{    //row selected
    strTableSelectedFile = [marrOfFiles objectAtIndex:indexPath.row];  //find what file was selected
    
    //enable the load and delete buttons when a file has been pressed in the table
    btnLoadFile.enabled = true;
    btnLoadFile.alpha = 1;
    btnDeleteFile.enabled = true;
    btnDeleteFile.alpha = 1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    UITableViewCell *cellTable;
    UILabel *labelCell;
    cellTable = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
    
    if (cellTable == nil)   //set up the cell in the table
    {
        cellTable = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"Cell"];
        labelCell                 = [[UILabel alloc] initWithFrame:CGRectZero];
        labelCell.font            = [UIFont fontWithName:@"Arial" size:15];
        labelCell.numberOfLines   = 0;
        labelCell.tag             = 1;
        labelCell.backgroundColor = [UIColor clearColor];
        
        cellTable.selectionStyle = UITableViewCellSelectionStyleBlue;
        [[cellTable contentView] addSubview:labelCell];
    }
    
    NSString *strTextCom = [marrOfFiles objectAtIndex:[indexPath row]];
    CGSize constraint = CGSizeMake(184.0, 300.0);
    CGSize size = [strTextCom sizeWithFont:[UIFont fontWithName:@"Arial" size:15] constrainedToSize:constraint];
    
    if (!labelCell){
        labelCell = (UILabel*)[cellTable viewWithTag:1];
    }
    
    [labelCell setText:strTextCom];
    [labelCell setFrame:CGRectMake(10, 10, 184.0, MAX(size.height, 44.0f))];
    return cellTable;
}
//End required table view methods





//UICollectionView methods
#pragma mark - UICollectionView
- (NSInteger)collectionView:(UICollectionView *)view numberOfItemsInSection:(NSInteger)section {
    return 16;  //16 drums per sections
}

- (NSInteger)numberOfSectionsInCollectionView: (UICollectionView *)collectionView {
    return 2;   //2 drums
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)cView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    drumCell *cell = (drumCell *)[cvDrumGrid dequeueReusableCellWithReuseIdentifier:@"Cell" forIndexPath:indexPath];    //find the cell the user clicked
    return cell;    //return the cell the user clicked
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath{
    drumCell *cell = (drumCell *)[cvDrumGrid cellForItemAtIndexPath:indexPath];
    int drumSelected = indexPath.section;
    
    //Change colour of views if the user clicked the specific view
    switch (drumSelected) {
        case 0: //drum 1
            if([[marrDrum1 objectAtIndex:indexPath.row] integerValue] == 0){    //if cell selected at beat i then play drum one
                [marrDrum1 replaceObjectAtIndex:indexPath.row withObject:@"1"]; //change value in drum array
                cell.drumView.BackgroundColor = [UIColor redColor];     //change background colour of cell
            }else{ //cell de-selected
                [marrDrum1 replaceObjectAtIndex:indexPath.row withObject:@"0"]; //change value in array
                cell.drumView.BackgroundColor = [UIColor whiteColor];   //change colour back to normal
            }
            break;
            
        case 1: //drum 2
            if([[marrDrum2 objectAtIndex:indexPath.row] integerValue] == 0){    //if cell selected at beat i then play drum one
                [marrDrum2 replaceObjectAtIndex:indexPath.row withObject:@"1"];
                cell.drumView.BackgroundColor = [UIColor redColor];
            }else{
                [marrDrum2 replaceObjectAtIndex:indexPath.row withObject:@"0"];
                cell.drumView.BackgroundColor = [UIColor whiteColor];
            }
            break;
            
        default:
            break;
    }
}
//End required UICollectionView methods

-(void) resetCollectionRow:(int)row resetCollectionSection:(int)section{    //reset view in the uicollectionview
    NSIndexPath *indexPath= [NSIndexPath indexPathForRow:row inSection:section];    //get the index of the view to reset
    drumCell *cell = (drumCell *)[cvDrumGrid cellForItemAtIndexPath:indexPath]; //create a new drumCell for that index
    cell.drumView.BackgroundColor = [UIColor whiteColor];   //set the colour to be white
}
@end