//
//  ViewController.m
//
//  Created by Steffan Walters on 12/11/2012.
//  Copyright (c) 2012 Steffan Walters. All rights reserved.
//

#import "ViewController.h"
#import <CoreMIDI/CoreMIDI.h>
#import <AudioToolbox/AudioToolbox.h>
#import "effectView.h"

//Tutorial from Making Musical Apps book followed to load a Pure Data patch into the xCode application
//MIDI input code adapted from Learning Core Audio - A hands-On guide to Audio Programming for Mac and iOS - chapter 11 Core MIDI

@interface ViewController ()

@property (strong, nonatomic) NSArray *arrOfFiles;  //array to store file names

@property (strong, nonatomic) AVAudioPlayer *player;    //audio players for each drum
@property (strong, nonatomic) AVAudioPlayer *player2;

@property (strong, nonatomic) NSString *strTableSelectedFile;   ////selected file from table

@property(strong,nonatomic) effectView *viewEffect; //Effects view subview, used for adding point onto the screen

-(void) appendToTextView: (NSString*) moreText; //append text to the tvLabelOutput text view

@end



@implementation ViewController

//see interface for brief description
@synthesize tvLabelOutput, btnPlayRecording;
@synthesize player, player2;
@synthesize arrOfFiles;
@synthesize btnPlayDrums;
@synthesize strTableSelectedFile;
@synthesize tvFileLoad;
@synthesize viewEffect;
@synthesize sliTremelo, sliTremeloRange, lblTremeloRange, lblTremeloValue;
@synthesize btnStopDrums;
@synthesize sliAattack, sliDecay, sliSustain, sliRelease;
@synthesize btnChords, btnDiminish, btnRandom, btnUp, sgSoundType;
@synthesize viewEffectSuper;

#pragma mark - C Variable definitions/declerations
bool boolUpArpegio = NO;        //up arpegio on or off
bool boolRandomArpegio = NO;    //random arpegio on or off
bool boolDiminished = NO;       //diminished effect on or off
bool boolChord = NO;
bool boolRecord = NO;           //recording or not
bool boolDrumLoop = true;       //drums playing or not

NSDate *dateStart;      //date variable - used for recording (getting the correct on/off time of note)

int intTremeloValue = 0;        //used for tremelo effect
int intTremeloRange = 100;      //used for tremelo effect

struct structNoteRecord{        //a struct to hold recorded note information (linked list)
    int note;               //note
    int volume;
    float OnOfftime;        //play/release time
    bool onOrOff;           //note released or played
    bool diminish;          //diminish effect is on?
    bool arpeggio;          //up arpeggio effect is on?
    bool arpeggioRand;      //random arpeggio effect is on?
    bool chord;             //chord effect is on?
    int soundType;          //type of sound while recorded
    struct structNoteRecord * next;     //next node in list
};

typedef struct structNoteRecord item;      //typedef of struct

item * curr, *head, *tail;      //curr, head and tail nodes

//ENVELOPE CONTAINERS
int intSustain = 60;
int intAttack = 1000;
int intDecay = 1000;
int intRelease = 1000;
//END OF ENVELOPE

int intSoundType = 0;   //int to hold what sound type is to be played, 0 for normal, 1 for sawtooth, 2 for square

#pragma mark - View lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    head = NULL;    //set the head of linked list to be NULL
    tail = NULL;     //set the taile of linked list to be NULL
    
    //set up dispatcher/patch for normal midi play
    //Tutorial from Making Musical Apps book followed to load a Pure Data patch into the xCode application
    normalDispatcher = [[PdDispatcher alloc]init];
    [normalDispatcher addListener:self forSource:@"pitch"];
    [PdBase setDelegate:normalDispatcher];
    normalPatch = [PdBase openFile:@"simple_synth.pd" path:[[NSBundle mainBundle] resourcePath]];
    
    [PdBase sendFloat:0 toReceiver:@"ampMod"];  //ensure no effect is on at the start
    [PdBase sendFloat:0 toReceiver:@"temp"];
    
    // Set up avaudioplayers for the drums
    NSString *soundFilePath = [[NSBundle mainBundle] pathForResource:@"drum1" ofType:@"wav"];
    NSURL *soundFileURL = [NSURL fileURLWithPath:soundFilePath];
    player = [[AVAudioPlayer alloc] initWithContentsOfURL:soundFileURL error:nil];
    player.numberOfLoops = 0;
    [player prepareToPlay];
    
    NSString *soundFilePath2 = [[NSBundle mainBundle] pathForResource:@"drum2" ofType:@"wav"];
    NSURL *soundFileURL2 = [NSURL fileURLWithPath:soundFilePath2];
    player2 = [[AVAudioPlayer alloc] initWithContentsOfURL:soundFileURL2 error:nil];
    player2.numberOfLoops = 0;
    [player2 prepareToPlay];
    
    [self fileSetup];   //call fileSetup to populate arrOfFiles with the files saved on the iPad
}

-(void)viewDidAppear:(BOOL)animated{    //called when view is transitioned to from another view
    [self fileSetup];   //call fileSetup to populate arrOfFiles with the files saved on the iPad
    [tvFileLoad reloadData];  //reload the data in the file table
}

- (void)viewDidUnload{
    [super viewDidUnload];
    [PdBase closeFile:normalPatch];
    [PdBase setDelegate:nil];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation{
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}





#pragma mark - Saved File Setup
-(void)fileSetup{
    //Find files in the directory folder of the app and put these contents in arrayoffiles
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSFileManager *filemgr = [NSFileManager defaultManager];
    arrOfFiles = [filemgr contentsOfDirectoryAtPath: [paths objectAtIndex:0] error: nil];
    
    if ([arrOfFiles count] == 0) {  //if there are no files then disable the play and stop buttons
        btnPlayDrums.enabled = false;
        btnPlayDrums.alpha = 0.5;
        btnStopDrums.enabled = false;
        btnStopDrums.alpha = 0.5;
    }
}




#pragma mark - Check Error C-Method For MIDI Setup
//Code used from Learning Core Audio - A hands-On guide to Audio Programming for Mac and iOS - chapter 11 Core MIDI
//assert there is no error
static void CheckError(OSStatus error, const char *operation){
    
    if(error==noErr){
        return;
    }
    
    char errorString[20];
    *(UInt32 *)(errorString + 1) = CFSwapInt32HostToBig(error);
    if (isprint(errorString[1]) && isprint(errorString[2]) && isprint(errorString[3]) && isprint(errorString[4])){
        //errorString[0] = errorString[5] = '/';
        //errorString[6] = '/0';
    }else{
        sprintf(errorString, "%d", (int)error);
    }
    exit(1);
}






#pragma mark - C-Method Used While Creating Client
//Code used from Learning Core Audio - A hands-On guide to Audio Programming for Mac and iOS - chapter 11 Core MIDI
void MyMIDINotifyProc(const MIDINotification *message, void *refCon){
    printf("MIDI Notify, messageId=%ld", message->messageID);
    ViewController *viewController = (__bridge ViewController *) refCon;
    [viewController appendToTextView:[NSString stringWithFormat:@"MIDI Notify, messageId=%ld \n", message->messageID]]; //log any message
}





#pragma mark - MIDI Input Recieve C-Method - Send Data to PD Patch
//Code to get MIDI data adapted from Learning Core Audio - A hands-On guide to Audio Programming for Mac and iOS - chapter 11 Core MIDI
static void MyMIDIReadProc(const MIDIPacketList *pktlist, void *refCon, void *connRefCon){
    
    ViewController *viewController = (__bridge ViewController *) refCon;    //get a reference to the view controller
    MIDIPacket *packet = (MIDIPacket *)pktlist->packet; //get the current packet
    
    for (int i = 0; i < pktlist->numPackets; i++) { //iterate over packets
        
        Byte midiStatus = packet->data[0] >> 4;     //get the status byte of the MIDI data
        Byte note = packet->data[1] & 0x7F;         //get the MIDI note number
        Byte velocity = packet->data[2] & 0x7F;     //get the velocity that the note was played
        if(midiStatus == 0x09){ //if a note is played
            
            //if the user is recording the input
            if(boolRecord){
                //find the time since the start of the dateStart Date (to ensure correct timing on playback)
                NSTimeInterval timeInterval = [dateStart timeIntervalSinceNow];
                
                //allocate space for now item object
                curr = (item *)malloc(sizeof(item));
                
                //set all parameters of the item object (or struct)
                curr -> note = (int)note;
                curr -> volume = (int)velocity;
                curr -> OnOfftime = timeInterval;
                curr -> onOrOff = true; //specifies note on
                curr -> soundType = intSoundType;
                curr -> diminish = boolDiminished;
                curr -> arpeggio = boolUpArpegio;
                curr -> arpeggioRand = boolRandomArpegio;
                curr -> chord = boolChord;
                
                //set the next item object to be null
                curr -> next = NULL;
                
                //if this is the first note recorded, then the head of the linked list is equal to null
                if (!head) {
                    head = curr;
                    tail = curr;
                }else{
                    tail -> next = curr;
                    tail = curr;
                }
                
            }
            
            if(boolDiminished){   //diminish play - send correct parameters to diminish synthesiser
                [PdBase sendFloat:note toReceiver:@"diminishedMidinote"];
                [PdBase sendFloat:velocity toReceiver:@"diminishedVolume"];
            }else if(boolRandomArpegio){    //up arpeggio play - send correct parameters to arpeggio synthesiser
                [PdBase sendFloat:velocity toReceiver:@"arpegioVolume"];
                [PdBase sendFloat:intSoundType toReceiver:@"arpegioSoundType"];
                [PdBase sendFloat:0 toReceiver:@"option"];  //0 specifies random arpeggio to be played
                [PdBase sendFloat:note toReceiver:@"arpegioMidinote"];
            }else if(boolUpArpegio){    //random arpeggio play - send correct parameters to arpeggio synthesiser
                [PdBase sendFloat:velocity toReceiver:@"arpegioVolume"];
                [PdBase sendFloat:1 toReceiver:@"option"];  //1 specifies up arpeggio to be played
                [PdBase sendFloat:intSoundType toReceiver:@"arpegioSoundType"];
                [PdBase sendFloat:note toReceiver:@"arpegioMidinote"];
                                
            }else{  //send correct parameters to the main synthesiser
                
                [PdBase sendFloat:velocity toReceiver:@"volume"];   
                [PdBase sendFloat:intSoundType toReceiver:@"soundType"];    //0 for normal, 1 for sawtooth, 2 for square
                
                if(boolChord){  //if the chord effect is on
                    [PdBase sendFloat:1 toReceiver:@"chord"];   //send a 1 to the PD patch to ensure the chord effect is played
                }else{
                    [PdBase sendFloat:0 toReceiver:@"chord"]; //ensure chord effect is not on unless stated
                }
                
                //send the envelope parameters
                [PdBase sendFloat:intAttack toReceiver:@"attack"];
                [PdBase sendFloat:intDecay toReceiver:@"decay"];
                [PdBase sendFloat:intSustain toReceiver:@"sustain"];
                [PdBase sendFloat:note toReceiver:@"midinote"];
                
                //send the modulating parameters
                if (intTremeloValue > 0) {
                    [PdBase sendFloat:intTremeloValue toReceiver:@"ampMod"];
                    [PdBase sendFloat:intTremeloRange toReceiver:@"temp"];
                    
                }                 
                
            }
            //debug statement
            [viewController appendToTextView:[NSString stringWithFormat:@"Note ON. Note=%d, Velocity=%d, ", note, velocity]];
            
        }else if(midiStatus == 0x08){      //if a note is released
            
            //if the user is recording the input
            if(boolRecord){
                //find the time since the start of the dateStart Date (to ensure correct timing on playback)
                NSTimeInterval timeInterval = [dateStart timeIntervalSinceNow];
                
                //allocate space for a new item object
                curr = (item *)malloc(sizeof(item));
                
                //set all parameters of the item object (or struct)
                curr -> note = (int)note;
                curr -> volume = 0;
                curr -> OnOfftime = timeInterval;
                curr->onOrOff = false;  //specifies note off
                curr -> soundType = intSoundType;
                curr -> diminish = boolDiminished;
                curr -> arpeggio = boolUpArpegio;
                curr -> arpeggioRand = boolRandomArpegio;
                curr -> chord = boolChord;
                //set the next object in the linked list to be null
                curr -> next = NULL;
                
                tail-> next = curr;
                tail = curr;

            }
            
            if (!boolDiminished) {  //if the diminished effect isn't played (it fades out for the diminished) then stop the playing of the note
                
                //send the required parameters to the PD patch for the stopping of a note
                
                //[PdBase sendBangToReceiver:@"stop"];
                [PdBase sendFloat:intRelease toReceiver:@"release"];
                [PdBase sendFloat:intSoundType toReceiver:@"soundType"];
                [PdBase sendFloat:note toReceiver:@"midiOffNote"];
                [PdBase sendBangToReceiver:@"arpegioStop"];              
            }
            //debug statement
            [viewController appendToTextView:[NSString stringWithFormat:@"Note Off. Note=%d", note]];
        }
        
        packet = MIDIPacketNext(packet);    //move on to the next packet, possibly falling out of the loop
    }
}





#pragma mark - Objective-C Methods for MIDI input set up
//Code used from Learning Core Audio - A hands-On guide to Audio Programming for Mac and iOS - chapter 11 Core MIDI
-(void) setupMIDI{
    MIDIClientRef client = NULL;    //create a session
    CheckError(MIDIClientCreate(CFSTR("Core MIDI to System Sound Demo"), MyMIDINotifyProc, (__bridge void *)(self), &client), "Couldn't create MIDI client");
    
    MIDIPortRef inPort;     //Input port to recieve MIDI data
    CheckError(MIDIInputPortCreate(client, CFSTR("Input port"), MyMIDIReadProc, (__bridge void *)(self), &inPort), "Couldn't create MIDI input port");
    
    unsigned long sourceCount = MIDIGetNumberOfSources();   //Get number of sources
    [self appendToTextView:[NSString stringWithFormat:@"%ld sources\n", sourceCount]];
    
    for(int i = 0; i < sourceCount; ++i){   //create an ebdpoint reference for each input device
        MIDIEndpointRef src = MIDIGetSource(i);
        CFStringRef endpointName = NULL;
        
        CheckError(MIDIObjectGetStringProperty(src, kMIDIPropertyName, &endpointName), "Couldn't get endpoint name");   //get the name of the device
        
        char endpointNameC[255];
        CFStringGetCString(endpointName, endpointNameC, 255, kCFStringEncodingUTF8);    
        
        NSString *temp = [[NSString alloc]initWithUTF8String:endpointNameC];   
        [self appendToTextView: [NSString stringWithFormat:@"  source %d: %@\n", i, temp]];
        
        CheckError(MIDIPortConnectSource(inPort, src, NULL), "Couldn't connect MIDI port");     //connect to the source
    }
}





#pragma mark - Setup Keyboard Input
- (IBAction)actionConnectKeyboard:(id)sender {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Set-up Keyboard" message:@"Please note that clicking this button twice will cause a failure" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Set-up Keyboard", nil];
    
    [alert show];   //create and display an alert asking if the user wants to set up the keyboard
}





#pragma mark - Text View Methods - Addd and Clear
//add to Text View - allows C methods to add to text view
-(void) appendToTextView: (NSString*) moreText {
	dispatch_async(dispatch_get_main_queue(), ^{
		tvLabelOutput.text = [NSString stringWithFormat:@"%@%@\n",
                              self.tvLabelOutput.text, moreText];   //add text to the text view
		[tvLabelOutput scrollRangeToVisible:NSMakeRange(tvLabelOutput.text.length-1, 1)];
	});
}

//Clear Text View
- (IBAction)actionClearText:(id)sender {
    tvLabelOutput.text = @"";   //clears the text view
}





#pragma mark - Diminished Effect
- (IBAction)actionDiminished:(id)sender {
    if (boolDiminished) {   //if the diminished effect is on
        boolDiminished = NO;    //disable the effect
        
        //Enable other interface objects
        btnChords.enabled = true;
        btnChords.alpha = 1;
        btnUp.enabled = true;
        btnUp.alpha = 1;
        btnRandom.enabled = true;
        btnRandom.alpha = 1;
        sgSoundType.enabled = true;
    } else {    //diminished effect not on
        boolDiminished = YES;   //enable diminished effect
        
        //Disable other interface objects (those which can't be used at the same time as this effect)
        btnChords.enabled = false;
        btnChords.alpha = 0.5;
        btnUp.enabled = false;
        btnUp.alpha = 0.5;
        btnRandom.enabled = false;
        btnRandom.alpha = 0.5;
        sgSoundType.enabled = false;
    }
}





#pragma mark - Chord Effect
- (IBAction)actionChord:(id)sender {
    if (boolChord) {   //if the chord effect is on
        boolChord = NO;    //disable the effect
        
         //Enable other interface objects
        btnDiminish.enabled = true;
        btnDiminish.alpha = 1;
        btnUp.enabled = true;
        btnUp.alpha = 1;
        btnRandom.enabled = true;
        btnRandom.alpha = 1;
        
    } else {    //chord effect not on
        boolChord = YES;   //enable diminished effect
        
        //Disable other interface objects (those which can't be used at the same time as this effect)
        btnDiminish.enabled = false;
        btnDiminish.alpha = 0.5;
        btnUp.enabled = false;
        btnUp.alpha = 0.5;
        btnRandom.enabled = false;
        btnRandom.alpha = 0.5;
        
    }
}





#pragma mark - Arpegio Effects
//Random Arpegio
- (IBAction)actionRandomArpegio:(id)sender {
    if (boolRandomArpegio) {    //if the random effect is on
        boolRandomArpegio = NO; //disable effect
        
         //Enable other interface objects
        btnDiminish.enabled = true;
        btnDiminish.alpha = 1;
        btnUp.enabled = true;
        btnUp.alpha = 1;
        btnChords.enabled = true;
        btnChords.alpha = 1;
        
        [PdBase sendBangToReceiver:@"arpegioStop"]; //send stop to PD reciever (in case it is still playing)
    }else{      //random effect not on
        boolRandomArpegio = YES;    //enable effect
        
        //Disable other interface objects (those which can't be used at the same time as this effect)
        btnDiminish.enabled = false;
        btnDiminish.alpha = 0.5;
        btnUp.enabled = false;
        btnUp.alpha = 0.5;
        btnChords.enabled = false;
        btnChords.alpha = 0.5;
    }
}

//Up Arpegio
- (IBAction)actionUpArpegio:(id)sender {
    if (boolUpArpegio) {    //if the up arpegio effect is on
        boolUpArpegio = NO; //disable effect
        
         //Enable other interface objects
        btnDiminish.enabled = true;
        btnDiminish.alpha = 1;
        btnChords.enabled = true;
        btnChords.alpha = 1;
        btnRandom.enabled = true;
        btnRandom.alpha = 1;

        [PdBase sendBangToReceiver:@"arpegioStop"]; //send stop to PD reciever (in case it is still playing)
    }else{      //random effect not on
        boolUpArpegio = YES;    //enable effect
        
        //Disable other interface objects (those which can't be used at the same time as this effect)
        btnDiminish.enabled = false;
        btnDiminish.alpha = 0.5;
        btnChords.enabled = false;
        btnChords.alpha = 0.5;
        btnRandom.enabled = false;
        btnRandom.alpha = 0.5;
    }
}





#pragma mark - Record Input and Playback Input
//Record
- (IBAction)actionRecord:(id)sender {
    if (!boolRecord) {  //if not recording - then begin recording
        btnPlayRecording.hidden = YES;  //hide the btnPlayRecording button
        boolRecord = YES;   //set boolRecord to true so that input will be recorded in the MyMIDIReadProc C-Method
        
        head = NULL;
        
        dateStart = [NSDate date];  //start the times that will be used to get the time that the notes are played
    }else{  //else recording - then stop recording
        btnPlayRecording.hidden = NO;   //show the play button and set the boolean variable used for recording
        boolRecord = NO;
    }
}

//Playback
- (IBAction)actionPlaybackRecording:(id)sender {
    
    curr = head;    //set the current element of the list to be the head
    
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT,(unsigned long)NULL), ^(void) {    //do another task - allows other tasks to be performed while the code below is executed
        NSDate *compare = [NSDate date];    //NSDate variable to compare the time to that of the notes stored in the linked list
        
        while (curr) {  //while there is a node
            int note = curr->note;  //get note
            int velocity = curr -> volume;
            bool boolOnOff = curr->onOrOff; //get it's status (on or off)
            double noteTime= curr->OnOfftime;   //get the time it is meant to be played/ stopped
            int intTypeOfSound = curr -> soundType; //get the sound type
            //Get the state of the effects
            bool arpeggio = curr -> arpeggio;
            bool arpeggioRand = curr -> arpeggioRand;
            bool diminish = curr -> diminish;
            bool chord = curr -> chord;
            
            
            NSTimeInterval timeInterval2 = [compare timeIntervalSinceNow];  //compare the start time(compare) with the time now
            noteTime = round (noteTime * 100) / 100.0; //round the note time to 2 decimal places
            double interval = round (timeInterval2 * 100) / 100.0;  //round the interval time to 2 decimal places
            
            if (interval == noteTime) { //if the interval and the note time are the same then a note is to be played
                
                if(diminish){   //if the diminished effect is applied, send correct parameters to the diminished synthesiser
                    [PdBase sendFloat:note toReceiver:@"diminishedMidinote"];
                    [PdBase sendFloat:velocity toReceiver:@"diminishedVolume"];
                }else if(arpeggio){ //if the up arpeggio effect is applied, send correct parameters to the diminished synthesiser
                    [PdBase sendFloat:intTypeOfSound toReceiver:@"arpegioSoundType"];
                    if (boolOnOff == true) {    //note is to be played
                        [PdBase sendFloat:velocity toReceiver:@"arpegioVolume"];
                        [PdBase sendFloat:1 toReceiver:@"option"];
                        
                        [PdBase sendFloat:note toReceiver:@"arpegioMidinote"];
                    }else{  //note is to be stopped
                        [PdBase sendBangToReceiver:@"arpegioStop"];
                    }
                }else if(arpeggioRand){ //if the random arpeggio effect is applied, send correct parameters to the diminished synthesiser
                    if (boolOnOff == true) {    //note is to be played
                        [PdBase sendFloat:velocity toReceiver:@"arpegioVolume"];
                        [PdBase sendFloat:0 toReceiver:@"option"];
                        [PdBase sendFloat:intTypeOfSound toReceiver:@"arpegioSoundType"];
                        [PdBase sendFloat:note toReceiver:@"arpegioMidinote"];
                    }else{  //note is to be stopped
                        [PdBase sendBangToReceiver:@"arpegioStop"];
                    }
                }else{  //else send correct parameters to the main synthesizer

                   [PdBase sendFloat:intTypeOfSound toReceiver:@"soundType"];   //send the sound type

                    
                    if(chord){  //check if the chord effect is applied
                        [PdBase sendFloat:1 toReceiver:@"chord"];   //send if chord effect is applied
                    }else{
                        [PdBase sendFloat:0 toReceiver:@"chord"]; //ensure chord effect is not on unless stated
                    }
                    
                    if (boolOnOff == true) {    //note is to be played
                        [PdBase sendFloat:velocity toReceiver:@"volume"];    //send volume to correct PD reciever
                        [PdBase sendFloat:note toReceiver:@"midinote"]; //send note to correct PD reciever
                    }else{  //note is to be stopped
                        [PdBase sendFloat:note toReceiver:@"midiOffNote"];  //send note to correct PD reciever
                    }
                }
                curr = curr->next;  //move to next node in the list
            }
        }
    });
}





#pragma mark - Drum Methods - Play and Stop
//Play Drums
- (IBAction)actionPlayDrums:(id)sender {
    boolDrumLoop = true;    //keep playing - condition for while loop
    btnPlayDrums.enabled = false;  //disable play button
    btnPlayDrums.alpha=0.5;
    
    btnStopDrums.enabled = true;    //enable the stop button
    btnStopDrums.alpha=1;
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);   //path to documents directory
    NSString *filePath = [[paths objectAtIndex:0]stringByAppendingPathComponent:strTableSelectedFile];   //path of file
    
    NSArray *fileArray = [[NSArray alloc]initWithContentsOfFile:filePath];      //array of saved arrays
    NSArray *arrVolDrum = [[NSArray alloc] initWithArray:fileArray[0]];         //volume array containing bpm and volumes
    
    NSString *bpmVal = [arrVolDrum objectAtIndex:0];        //the bpm value of the track
    NSString *drum1Volume = [arrVolDrum objectAtIndex:1];   //volume for drum 1
    NSString *drum2Volume = [arrVolDrum objectAtIndex:2];   //volume for drum 2
    
    float floatBeatsPerMin = 60 / [bpmVal floatValue];      //set sleep time for the track as 60 seconds / bpm
    
    NSArray *drum1 = [[NSArray alloc] initWithArray:fileArray[1]]; //array of drum 1
    player.volume = [drum1Volume floatValue];   //setting the volume of drum 1
    
    NSArray *drum2 = [[NSArray alloc] initWithArray:fileArray[2]];  //array of drum 2
    player2.volume = [drum2Volume floatValue];  //setting the volume of drum 2
    
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT,(unsigned long)NULL), ^(void) {    //do another task - allows other tasks to be performed while the code below is executed
        do {
            for (int i = 0; i < 16; i++) {   //16 beats
                
                //if the first drum is selected at the current beat, then play it
                if([[drum1 objectAtIndex:i] integerValue] == 1){
                    NSLog(@"i = %i, 1111 = %@", i ,[drum1 objectAtIndex:i]);
                    [player play];
                }
                
                //if the second drum is selected at the current beat, then play it
                if([[drum2 objectAtIndex:i] integerValue] == 1){
                    NSLog(@"i = %i, 2222 = %@", i ,[drum2 objectAtIndex:i]);
                    [player2 play];
                }
                
                [NSThread sleepForTimeInterval:floatBeatsPerMin];   //sleep for certain period of time to ensure beat (e.g. if beats per minutes is 60 then it would sleep for a second, 120 sleep for 0.5 etc.)
                
                [player stop];  //if players are still playing, then stop them (in case sample is longer than necessary)
                [player2 stop];
                
                if (!boolDrumLoop) {   //if booldrumloop set to false it will stop playing (stop button pressed) - break out of loop
                    break;
                }
            }
            
        } while (boolDrumLoop); //continue until user clicks the stop button
    });
}

//Stop Drums
- (IBAction)actionStopDrums:(id)sender {
    btnPlayDrums.enabled = true;   //enable the play button
    btnPlayDrums.alpha=1;
    btnStopDrums.enabled = false;   //diable the stop button
    btnStopDrums.alpha=0.5;
    boolDrumLoop = false;   //stops the drums by setting boolInfiniteLoop to false which breaks the while loop in the actionPlayDrums method
}





#pragma mark - Sliders - Tremelo
- (IBAction)actionsliTremeloChange:(id)sender {
    intTremeloValue = (int)[sliTremelo value];  //retrieve value from the sliTremelo slider
    lblTremeloValue.text = [NSString stringWithFormat:@"%i", intTremeloValue];  //set the value of the label
    
    //update the effect view
    [self.viewEffect removeFromSuperview];
    CGRect rect = CGRectMake(intTremeloValue*40, intTremeloRange*2, 10, 10);   //create the graphics point
    viewEffect = [[effectView alloc] initWithFrame:rect];   //allocate and initialise the subview with the frame of rect
    [self.viewEffectSuper addSubview:viewEffect];   //add viewEffect onto viewEffectSuper as a subview
    
    [PdBase sendFloat:intTremeloValue toReceiver:@"ampMod"];    //send update to PD patch
}

- (IBAction)actionsliTremeloRangeChange:(id)sender {
    intTremeloRange = (int)[sliTremeloRange value]; //retrieve the value from the rsliTremeloRange slider
    lblTremeloRange.text = [NSString stringWithFormat:@"%i", intTremeloRange];  //set the value of the label
    
    //update the effect view
    [self.viewEffect removeFromSuperview];
    CGRect rect = CGRectMake(intTremeloValue*40, intTremeloRange*2, 10, 10);   //create the graphics point
    viewEffect = [[effectView alloc] initWithFrame:rect];   //allocate and initialise the subview with the frame of rect
    [self.viewEffectSuper addSubview:viewEffect];   //add viewEffect onto viewEffectSuper as a subview
    
    [PdBase sendFloat:intTremeloRange toReceiver:@"temp"];  //send update to PD patch

}

#pragma mark - Gestures - Tremelo
- (IBAction)actionTapGesture:(id)sender {
    CGPoint touchPoint = [sender locationInView:viewEffectSuper];   //Retrieve the point of the tap
    float xPos = touchPoint.x;  //store x value of point
    float yPos = touchPoint.y;  //store y value of point
    tvLabelOutput.text=[NSString stringWithFormat:@"%@ touched at: %@", tvLabelOutput.text, NSStringFromCGPoint(touchPoint)];   //output point
    
    [self.viewEffect removeFromSuperview];  //remove any subview from the view (remove previous point graphics)
    
    //for tremelo effect
    //ensure that the tap is within the boundary
    if (xPos <0) {
        xPos =0;
    }else if(xPos > 400){
        xPos = 400;
    }
    
    if (yPos <0) {
        yPos =0;
    }else if (yPos > 400){
        yPos = 400;
    }

    CGRect rect = CGRectMake(xPos, yPos, 10, 10);   //create the graphics point
    viewEffect = [[effectView alloc] initWithFrame:rect];   //allocate and initialise the subview with the frame of rect
    [self.viewEffectSuper addSubview:viewEffect];   //add viewEffect onto viewEffectSuper as a subview
    
    xPos = xPos / 40;   //divide by 40 to get the value that will be sent to the PD patch
    intTremeloValue = xPos; //set the container for the tremelo valuy
    sliTremelo.value = intTremeloValue; //update tremelo slider
    lblTremeloValue.text = [NSString stringWithFormat:@"%i", intTremeloValue];  //set the value of the label
    
    yPos = yPos / 2;    //divide by 40 to get the value that will be sent to the PD patch
    intTremeloRange = yPos; //set the container for the tremelo range
    sliTremeloRange.value = intTremeloRange; //update range slider
    lblTremeloRange.text = [NSString stringWithFormat:@"%i", intTremeloRange];  //set the value of the label
    
    [PdBase sendFloat:intTremeloValue toReceiver:@"ampMod"];    //send tremeloValue to correct PD reciever
    [PdBase sendFloat:intTremeloRange toReceiver:@"temp"];      //send tremeloRange to correct PD reciever
}

- (IBAction)actionPanGesture:(id)sender { //see above for comments (actionTapGesture) - this method is for a pan gesture not just a single tap
    CGPoint touchPoint = [sender locationInView:viewEffectSuper];
    float xPos = touchPoint.x;
    float yPos = touchPoint.y;
    tvLabelOutput.text=[NSString stringWithFormat:@"%@ touched at: %@", tvLabelOutput.text, NSStringFromCGPoint(touchPoint)];
    
    [self.viewEffect removeFromSuperview];
    
    //For tremelo
    if (xPos <0) {
        xPos =0;
    }else if(xPos > 400){
        xPos = 400;
    }
    
    if (yPos <0) {
        yPos =0;
    }else if (yPos > 400){
        yPos = 400;
    }

    CGRect rect = CGRectMake(xPos, yPos, 10, 10);
    viewEffect = [[effectView alloc] initWithFrame:rect];
    [self.viewEffectSuper addSubview:viewEffect];

    xPos = xPos / 40;
    intTremeloValue = xPos;
    sliTremelo.value = intTremeloValue;
    lblTremeloValue.text = [NSString stringWithFormat:@"%i", intTremeloValue];
    
    yPos = yPos / 2;
    intTremeloRange = yPos;
    sliTremeloRange.value = intTremeloRange; 
    lblTremeloRange.text = [NSString stringWithFormat:@"%i", intTremeloRange];

    [PdBase sendFloat:intTremeloValue toReceiver:@"ampMod"];
    [PdBase sendFloat:intTremeloRange toReceiver:@"temp"];
}





#pragma mark - Sound Type Changed
- (IBAction)sgSoundTypeChange:(id)sender {
    intSoundType = [sender selectedSegmentIndex];   //get index of segment control - normal 0 - sawtooth 1 - square 2
    [PdBase sendBangToReceiver:@"stopAll"]; //send a stop signal to the PD patch to ensure that nothing playes after the change of sound type
}





#pragma mark - Envelope parameters changed actions
- (IBAction)actionSustainChange:(id)sender {
    int temp = (int)[sliSustain value]; //get value of sliSustain slider
    intSustain = temp;  //store value in intSustain
}

- (IBAction)actionAttackChange:(id)sender { //see actionSustainChange for comments
    int temp = (int)[sliAattack value];
    intAttack = temp;
}

- (IBAction)actionDecayChange:(id)sender {  //see actionSustainChange for comments
    int temp = (int)[sliDecay value];
    intDecay = temp;
}

- (IBAction)actionReleaseChange:(id)sender {    //see actionSustainChange for comments
    int temp = (int)[sliRelease value];
    intRelease = temp;
}
#pragma mark - End Envelope





#pragma mark - alert view method call
//called after button on alert view is pressed
-(void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex{
    if (([[alertView title] isEqual:@"Set-up Keyboard"]) && (buttonIndex == 1)) {   //if it is the setup keyboard alert view and they chose to set up the keyboard
        [self setupMIDI];   //call the method to set up the connection
        [self appendToTextView: [NSString stringWithFormat:@"Connection Established(If keyboard connected)"]];  //print to the screen that a connection was established
    }else{
        [self appendToTextView: [NSString stringWithFormat:@"Connection Cancelled"]];   //if there was more than one UIAlertView this message would need to be removed as it would display for the other UIAlertView when a button is clicked
    }
}





#pragma mark Table View Data Source Methods
- (NSInteger)tableView:(UITableView*)tableView numberOfRowsInSection:(NSInteger)section
{
	return [arrOfFiles count];  //number of rows is equal to the number of items that have been saved (in arrOfFiles)
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView;
{
	return 1;   //one section in the table
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    //row selected
    
    strTableSelectedFile = [arrOfFiles objectAtIndex:indexPath.row];    //find what file was selected
    
    //enable the load and delete buttons when a file has been pressed in the table
    btnPlayDrums.enabled = true;
    btnPlayDrums.alpha = 1;
    //btnStopDrums.enabled = true;
    //btnStopDrums.alpha = 1;
    boolDrumLoop = false;
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cellTable;
    UILabel *labelCom;
    
    cellTable = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
    
    if (cellTable == nil)   //set up the cell in the table
    {
        cellTable = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"Cell"];
        
        labelCom                 = [[UILabel alloc] initWithFrame:CGRectZero];
        labelCom.font            = [UIFont fontWithName:@"Arial" size:15];
        labelCom.numberOfLines   = 0;
        labelCom.tag             = 1;
        labelCom.backgroundColor = [UIColor clearColor];
        
        cellTable.selectionStyle = UITableViewCellSelectionStyleBlue;
        [[cellTable contentView] addSubview:labelCom];
    }
    
    NSString *strTextCom = [arrOfFiles objectAtIndex:[indexPath row]];  //get the name of the file for the table
    
    CGSize constraint = CGSizeMake(184.0, 300.0);   //size of the cell
    CGSize size = [strTextCom sizeWithFont:[UIFont fontWithName:@"Arial" size:15] constrainedToSize:constraint];
    
    if (!labelCom){
        labelCom = (UILabel*)[cellTable viewWithTag:1];
    }
    
    [labelCom setText:strTextCom];
    [labelCom setFrame:CGRectMake(10, 10, 184.0, MAX(size.height, 44.0f))];
    
    return cellTable;
    
}

@end






